"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
exports.transform = (split, transformation, key) => {
    const parts = split(key);
    return transformation(parts);
};
exports.toCamelCase = (split, key) => {
    return exports.transform(split, (parts) => {
        const allButFirst = lodash_1.map(lodash_1.slice(parts, 1), (str) => lodash_1.capitalize(str));
        return [parts[0], ...allButFirst].join('');
    }, key);
};
exports.toPascalCase = (split, key) => {
    return exports.transform(split, (parts) => {
        const capitalized = lodash_1.map(parts, (str) => lodash_1.capitalize(str));
        return lodash_1.join(capitalized, '');
    }, key);
};
exports.fromSnakeCaseToCamelCase = (key) => {
    return exports.toCamelCase((sourceKey) => sourceKey.split('_'), key);
};
exports.fromSnakeCaseToPascalCase = (key) => {
    return exports.toPascalCase((sourceKey) => sourceKey.split('_'), key);
};
exports.transformFromCamelCaseToSnakeCase = (object) => {
    return exports.transformFromTo((key) => key.split(/(?=[A-Z])/), (parts) => lodash_1.join(lodash_1.map(parts, (p) => lodash_1.lowerCase(p)), '_'), object);
};
exports.transformFromSnakeCaseToCamelCase = (object) => {
    return transformKeys(exports.fromSnakeCaseToCamelCase, object);
};
exports.transformFromTo = (sourceForm, transformation, object) => {
    const curriedTransform = lodash_1.curry(exports.transform);
    return transformKeys(curriedTransform(sourceForm, transformation), object);
};
function transformKeys(transformation, object) {
    const curriedSelf = lodash_1.curry(transformKeys);
    if (lodash_1.isArray(object)) {
        return lodash_1.map(object, curriedSelf(transformation));
    }
    if (!lodash_1.isObjectLike(object)) {
        return object;
    }
    const objectKeys = lodash_1.keys(object);
    return lodash_1.reduce(objectKeys, (newObj, key) => {
        const newKey = transformation(key);
        const value = object[key];
        const newValue = transformKeys(transformation, value);
        newObj[newKey] = newValue;
        return newObj;
    }, {});
}
exports.transformKeys = transformKeys;
//# sourceMappingURL=transformKeys.js.map